@
*               This is a Gauss program to estimate a                  *
*         SNP-Poisson using Full Info with a dummy endog vble          *
*                                                                      *
This Version 8-11-00                                                   *
************************************************************************@

new;
clear guardame,guardame2,guardame3,gbeta,ggama,grho,gsigma,gpoli;

library optmum,dutil,pgraph;
#include optmum.ext;
optset;
#include inthermi.prg;
#include hermite.prg;
#include hessian.src;
#include sa.src;

nunodes=26; /* Enter here the number of nodes of the integral */
{ nodes, wei }=hermite(nunodes);

@----------------------- Files names ------------------------------------@

fioutput = "fk2satz.out";
ficoef = "fk2sactz.fmt";
fimvc = "fk2samtz.fmt";


@---------------------- Polynomial degree--------------------------------@
k=2;
@ Remember to change constan? in the Integrame procedure     @

#include constan.prg; /* This reads the procedures where the constant
factor is computed. Do not Include it before fixing k */

@------------------------- Variables names ---------------------------------@
Namesy= { tottrips };

Namesx={WORKSCHL,
        HHMEM,
        DISTOCBD,
        AREASIZE,
        FULLTIME,
        DISTONOD,
        REALINC,
        WEEKEND };  /* do not include dummy */

Namesz ={WORKSCHL,
        HHMEM,
        DISTOCBD,
        AREASIZE,
        FULLTIME,
        ADULTS,
        DISTONOD,
        REALINC };

NamesD={ OWNVEH };

NamesOtros={ BETA0, ALFA0, RHO, SIGMA, A, RHOSIG };




@----------------------- Loading Variables -------------------------------@
dataset = "terzasc";
{y,borra1,borra2}=dgetvar(dataset,Namesy);
{x,borra1,borra2}=dgetvar(dataset,Namesx);
{d,borra1,borra2}=dgetvar(dataset,NamesD);
{z,borra1,borra2}=dgetvar(dataset,Namesz);

n=rows(x);
x=ones(n,1)~x;
z=ones(n,1)~z;
kx=cols(x)+1; /* this one is due to the dummy */
kz=cols(z);


@------------------- Comienza la ejecucion de resultados -----------------@

cls;
output file = ^fioutput reset;

PRINT "
--------------------- Statistics -------------------
";
print;
call dstat(dataset,0);
print;
"Estimation with nodes:";;nunodes;
"";
"Estimation using a Series Expansion over a Poisson ";
"";
cls;

@---------------------- Globales en los procedures ------------------------@
cln=lnfact(y)+0.5*ln(pi); @this is a global in the likelihood@

a0=1;                  @constante del polinomio@
_opgtol=1e-7;
_opalgr=2;
_opgdmd=0;
_oprteps=1e-2;
__title="Direct Minimization of Minus the ML";
__output=2;


break_points={0,1,2,3,4,5,6};

nomsnp= 0 $+ "A" $+ ftocv(seqa(1,1,k),1,0);

_opparnm=nomsnp|NamesOtros[1]
    |namesx
    |namesd
    |namesotros[2]
    |namesz
    |NamesOtros[5:6];


add=1;

@------------------------ Initial Conditions -----------------------------@
theta0=rndu(k+kx+kz+2,1)*0.1;
theta0={ 0.05, 0.23, -2.49, -0.41, 0.18, -0.04, 0.08, 0.3, 0.3, 0.11, -0.11,
2.3, -0.5, 0.31, 0.05, 0.64, -0.25, 1, 0.25, 0.05, 0.34, -0.53, 0.43 };

@----------- Parametros para Simulated Annealing ------------------------@

Maxevl=1000000000; @Maximum value functions evaluations@
T=1;               @INITIAL TEMPERATURE@
rt=0.70;      @Temperature Reduction Factor. 0.85 is suggested@

Vm={
       };

Vm=Vm';

TolCri=1e-7;  @(Called Eps) Tolerance Criteria. 1e-8 suggested@
Ns=20;        @Number of cycles to adjust VM. 20 is suggested@
Nt=90;       @Suggested=maxc(100,5*num_of_param)@
Iprint=1;     @Controls Output of the Algorithm@
Neps=4;
C=2*ones(k+kx+kz+2,1); @Step Length Adjustment. 2 is suggested@

max=0;        @0 indicates minimization. We use it because we have -LogL @

ub=10*ones(k+kx+kz+2,1);             @Upper bound@
lb=-ub;                              @Lower bound@

@-------------- Minimizacion por Simulated Annealing ---------------------@


{thetaest,nf,nacc,nfcnev,nobds,ier,t,vm}=
  sa(&mle,theta0,max,rt,TolCri,ns,nt,neps,maxevl,lb,ub,c,iprint,t,vm);


@----------------------- Anlisis del Gradiente -------------------------@
add=1;

"The SA point is";
thetaest;
save prov=thetaest;

"---The Gradient of the function, at the resulting points from SA is ----";
gradp(&mle,thetaest);
"________________________________________________________________________";


@---------------------------- Minimizacion -------------------------------@

theta0=thetaest; /* We start with the SA result */
{thetaest,nf,g,retcode}=optprt(optmum(&mle,theta0));

if retcode ne 0;
print " WARNING, RETCODE IS DIFFERENT FROM ZERO ";
endif;

@--------------- This recuperates some parameters ------------------------@

a=thetaest[kx+kz+2+k];
b=thetaest[kx+kz+1+k];
sigpre=sqrt(a^2+b^2); @real sigma@
rhopre=b/sigpre;  @ real rho@
thetapre=trimr(thetaest,0,2)|rhopre|sigpre;
deltajac=((a/sqrt(a^2+b^2))~(b/sqrt(a^2+b^2)))|
((-a*b*(a^2+b^2)^(-3/2))~((a^2+b^2)^(-1/2)-b^2*(a^2+b^2)^(-3/2)));

@--------------------- Computes VCV matrix -------------------------------@

add=0;
IT=gradp(&mle,thetaest);
guardame3=IT;
add=1;
IT=(1/n)*(IT'*IT);
HT=hessian(&mle,thetaest);
vcv=invpd(HT*inv(IT)*HT)/n;
vcvab=vcv[k+kx+kz+1:k+kx+kz+2,k+kx+kz+1:k+kx+kz+2];
vcvrs=deltajac*vcvab*deltajac';
std=sqrt(diag(vcv));
std=trimr(std,0,2)|sqrt(diag(vcvrs));

save ^ficoef =thetaest;
save ^fimvc =vcv;  @this save the vcm with original a and b@

@----------------------- Analisis de Autovalores -------------------------@
if minc(real(eig(ht)))<=0;
print "WARNING, Hessian with negative eigenvalues";
endif;

@----------------------- Presentacion de resultados -----------------------@

__fmtnv={"*.*lg" 15 8};
__fmtcv={"-*.*s" 12 8};

print"============================================================";
print"            Full Information Maximum Likelihood";
print"============================================================";
print"Parameter       Estimation     Std. Error         Z-Stat";
print"------------------------------------------------------------";
noms=nomsnp|NamesOtros[1]
    |namesx
    |namesd
    |namesotros[2]
    |namesz
    |NamesOtros[3:4];
call printfmt(noms~real(thetapre~std~(thetapre./std)),0~1~1~1);
print"------------------------------------------------------------";
print;

@-------------------- Estadisticos de bondad -----------------------@

print"Numero de parametros=";;k+kx+kz+2;
print"Log-Verosimilitud aproximada=";;-nf;
print"Criterio de Akaike (R)=";;2*n*nf+2*(k+kx+kz+2);
print"Criterio Consistente de Akaike (RS)=";;2*n*nf+(k+kx+kz+2)*(ln(n)+1);
print"Criterio de Schwarz BIC (R)=";;2*n*nf+(k+kx+kz+2)*ln(n);

@----------------- Calculamos la frecuencia esperada --------------------@

gbeta=thetapre[1:kx];
ggama=thetapre[kx+1:kx+kz];
grho=thetapre[kx+kz+1];
gsigma=thetapre[kx+kz+2];

arf=zeros(n,1);
maxy=maxc(y);
guardame=y;
pj=1;probip=zeros(n,1);

p=-1;do until p>=maxy-1; p=p+1; @this is for counts@


        y=p*ones(n,1);
        probip=probip+fajust(0)+fajust(1);

        if pj<=rows(break_points);
            if p==break_points[pj];
                arf=arf~probip;
                probip=zeros(n,1);
                pj=pj+1;
            endif;
        endif;

endo;

arf=arf[.,2:cols(arf)]~(1-sumc(arf'));
probest=meanc(arf);
y=guardame;

print;
print;
a1=rows(break_points)+1;
break_points=break_points|99*ones(1,1);
actual=counts(y,break_points)/n;

print "-----Counts------Real Number of Obs  --------Fitted Number of Obs";

fmt={ "*.*lf" 12 4,
      "*.*lf" 12 4,
      "*.*lf" 12 4 };
mask={ 1,1,1 };
mask=mask';
print "==================================================================";
results=break_points~actual~probest;
call printfm(results,mask,fmt);
print "------------------------------------------------------------------";
call printfm(sumc(results)',mask,fmt);


@------------------- Summary of goodness of fit --------------------------@

print"
--------------    Summary of goodness of fit measures    ------------------
";
print;
print "Summary measure of goodness of fit on probabilities is";
print "Taken from Cameron book, page 142-3, expression 5.33";
Print "The smallest, the best. It is always nonnegative";
sumc(abs((1/n)*(actual-probest)));


@--------------------------------------------------------------------------
    A continuacion calculamos el estadistico de ajuste (especificacion)
    del modelo tal y como aparece en el articulo de Andrews
    guardame3 contiene los gradientes en matriz NxK
---------------------------------------------------------------------------@

print"* Goodness of fit Test (Andrews, OPG version)";
print"  -------------------------------------------";
big_gamma=dummybr(y,break_points);
hache=(big_gamma-arf)~guardame3;
test=ones(n,1)'hache*invswp(hache'hache)*hache'ones(n,1);
print "Chi-Square goodness of fit= ";;test;
print "P-Value = ";;cdfchic(test,a1-1);

print"* Bondad de ajuste (no OPG)";
print"  -------------------------";

delta=0;
i=0;do until i>=n;i=i+1;

    delta=delta+guardame3[i,.]'*big_gamma[i,.];

endo;
delta=delta/n;
b=big_gamma-arf-(delta'*inv(ht)*guardame3')';
varia=(b'b)/n;
test=n*(actual-probest)'invpd(varia)*(actual-probest);
print "Chi-Square goddness of fit=";;test;
print "P-Value =";;cdfchic(test,a1-1);


@ -------- Grafico de las diferencias estandarizadas --------------- @

sigma=sqrt(diag(inv(varia)));
barra=sqrt(n)*(actual-probest)./sigma;
bar(0,barra);
print"Barras del grfico";
print"------------------";
barra;

dos move *.fmt matrices;
dos move *.out results;

output file=^fioutput off;
end;

/************************* FIN DEL PROGRAMA *******************************/

/*************************************************************************
                         Funcion Objetivo
**************************************************************************/

proc mle(theta);

        local loglike,i,xlim,llike;
        guardame=theta;
        gpoli=theta[1:k];
        gbeta=theta[k+1:k+kx];
        ggama=theta[k+kx+1:k+kx+kz];
        grho=theta[k+kx+kz+1];
        gsigma=theta[k+kx+kz+2];
        gsigma=sqrt(gsigma^2+grho^2);
        grho=grho/gsigma;
        loglike=ln(inthermi(&integrame,nodes,wei)+1e-7);
        loglike=loglike+ln(polin(y,a0|gpoli)^2)-cln;
        /* These last line is because these elements are not in the integral */
        /* It is important than the polinomial is squared before taking the log,
        otherwise it could take the log of a negative number */


    if add==1;

        retp(-meanc(loglike));
        else;
        retp(-loglike);
        endif;

    endp;

/***************************************************************************
    fajust: Calcula la probabilidad f(y,d|w) como funcion exclusiva de
            la variable binaria 'd'
***************************************************************************/

proc fajust(de);

        local prob,i,xlim,guardame2;

        gpoli=thetapre[1:k];
        gbeta=thetapre[k+1:k+kx];
        ggama=thetapre[k+kx+1:k+kx+kz];
        grho=thetapre[k+kx+kz+1];
        gsigma=thetapre[k+kx+kz+2];
        guardame2=d;
        d=de*ones(n,1);
        prob=inthermi(&integrame,nodes,wei);
        prob=prob.*(polin(y,a0|gpoli)^2)./(sqrt(pi)*y!);
        /* This is because these elements are out of the integral */
        d=guardame2;
        retp(prob);
endp;

/***********************************************************************
Integrame: Procedimiento para calcular el valor aproximado de la Log
            likelihood. Calcula la integral sobre epsilon para una
            observacion.
************************************************************************/

proc integrame(ksi);

    local cdfntau,tau,a1,a2,a3,total,coepol,eps,landa;

    eps=sqrt(2)*gsigma*ksi;
    tau=(z*ggama+sqrt(2)*grho*ksi)/sqrt(1-grho^2);
    cdfntau=cdfn(tau);
    a1=d.*cdfntau+(1-d).*(1-cdfntau);
    landa=(x~d)*gbeta+eps;
    a2=y.*landa-exp(landa); /* Do not use exp(-ksi^2) due to Hermite int */
    coepol=a0|gpoli;
    a3=exp(a2)./constan2(coepol,exp((x~d)*gbeta+eps)); @This must be changed@
                                                @according to poly degree@
    total=a1.*a3;
    /* polin(y,coepol)^2) and sqrt(pi)*y! are out of the integral */
    retp(total);
endp;

/**************************************************************************
    Procedimientos recursivos de Integrame.
**************************************************************************/


@----------- Calcula un polinomio en x de grado rows(coes) ---------------@

proc polin(x,coes);

    local poten;

        poten=potencias(x,rows(coes)-1);
        retp(poten*coes);

endp;

proc potencias(x,ka);

    local zeta,i;

        zeta=ones(rows(x),1);
        i=0;do until i>=ka;i=i+1;

            zeta=zeta~(x^i);

        endo;

    retp(zeta);

endp;

